import curses
try:
    from urllib import request
except ImportError:
    import urllib2 as request

from math import ceil

from music_scraper.threads import DownloadThread


class GUI:

    size_dict = {}
    url_dict = {}
    screen = None
    status = 'Scraping Music ... It might take some time.'
    strings = []
    pages = 0
    page = 1
    position = 1
    max_row = 0
    row_num = 0
    box = None
    high_light_text = None
    normal_text = None
    key = None
    run_download = False
    gui_stopped = False

    @staticmethod
    def init_display():
        if not GUI.gui_stopped:
            curses.noecho()
            curses.cbreak()
            curses.start_color()
            GUI.screen.keypad(1)
            curses.init_pair(1, curses.COLOR_BLACK, curses.COLOR_CYAN)
            GUI.high_light_text = curses.color_pair(1)
            GUI.normal_text = curses.A_NORMAL
            curses.curs_set(0)
            GUI.refresh_values()
            GUI.position = 1
            GUI.page = 1
            GUI.box = curses.newwin(GUI.max_row + 3, curses.COLS, 0, 0)
            GUI.box.addstr(1, 1, GUI.status, GUI.high_light_text)
            GUI.add_bottom_menus()
            GUI.screen.refresh()
            GUI.box.refresh()

    @staticmethod
    def refresh_values():
        if not GUI.gui_stopped:
            GUI.max_row = curses.LINES - 3
            GUI.row_num = len(GUI.strings)
            GUI.pages = int(ceil(GUI.row_num / GUI.max_row))

    @staticmethod
    def add_bottom_menus():
        GUI.box.addstr(curses.LINES - 1, 0, "ESC:Exit", GUI.high_light_text)
        GUI.box.addstr(curses.LINES - 1, curses.COLS // 2, "ENTR:Download", GUI.high_light_text)

    @staticmethod
    def on_key_down():
        if GUI.page == 1:
            if GUI.position < min(GUI.max_row, GUI.row_num):
                GUI.position += 1
            else:
                if GUI.pages > 1:
                    GUI.page += 1
                    GUI.position = 1 + (GUI.max_row * (GUI.page - 1))
        elif GUI.page == GUI.pages:
            if GUI.position < GUI.row_num:
                GUI.position += 1
        else:
            if GUI.position < GUI.max_row + (GUI.max_row * (GUI.page - 1)):
                GUI.position += 1
            else:
                GUI.page += 1
                GUI.position = 1 + (GUI.max_row * (GUI.page - 1))

    @staticmethod
    def on_key_up():
        if GUI.page == 1:
            if GUI.position > 1:
                GUI.position -= 1
        else:
            if GUI.position > (1 + (GUI.max_row * (GUI.page - 1))):
                GUI.position -= 1
            else:
                GUI.page -= 1
                GUI.position = GUI.max_row + (GUI.max_row * (GUI.page - 1))

    @staticmethod
    def on_key_left():
        if GUI.page > 1:
            GUI.page -= 1
            GUI.position = 1 + (GUI.max_row * (GUI.page - 1))

    @staticmethod
    def on_key_right():
        if GUI.page < GUI.pages:
            GUI.page += 1
            GUI.position = (1 + (GUI.max_row * (GUI.page - 1)))

    @staticmethod
    def on_key_enter():
        thread = DownloadThread(GUI.download_item)
        thread.start()

    @staticmethod
    def display_list():
        for i in range(1 + (GUI.max_row * (GUI.page - 1)), GUI.max_row + 1 + (GUI.max_row * (GUI.page - 1))):
            if GUI.row_num == 0:
                GUI.box.addstr(1, 1, GUI.status, GUI.high_light_text)
            else:
                if i + (GUI.max_row * (GUI.page - 1)) == GUI.position + (GUI.max_row * (GUI.page - 1)):
                    GUI.box.addstr(i - (GUI.max_row * (GUI.page - 1)), 2, str(i) + " - " + GUI.strings[i - 1],
                                   GUI.high_light_text)
                else:
                    GUI.box.addstr(i - (GUI.max_row * (GUI.page - 1)), 2, str(i) + " - " + GUI.strings[i - 1],
                                   GUI.normal_text)
                if i == GUI.row_num:
                    break

    @staticmethod
    def update_screen():
        if not GUI.gui_stopped:
            if GUI.key == curses.KEY_DOWN:
                GUI.on_key_down()
            elif GUI.key == curses.KEY_UP:
                GUI.on_key_up()
            elif GUI.key == curses.KEY_LEFT:
                GUI.on_key_left()
            elif GUI.key == curses.KEY_RIGHT:
                GUI.on_key_right()
            if GUI.key == ord("\n") and GUI.row_num != 0:
                GUI.on_key_enter()

            GUI.box.erase()
            GUI.display_list()
            GUI.add_bottom_menus()
            GUI.screen.refresh()
            GUI.box.refresh()

    @staticmethod
    def update_on_key():
        if not GUI.gui_stopped:
            GUI.key = GUI.screen.getch()
            while GUI.key != 27 or GUI.run_download:
                if ord('c') == GUI.key or ord('C') == GUI.key:
                    GUI.run_download = False
                GUI.update_screen()
                GUI.key = GUI.screen.getch()
            curses.endwin()

    @staticmethod
    def download_item():
        if GUI.run_download or GUI.gui_stopped:
            return
        GUI.run_download = True
        filename = GUI.strings[GUI.position - 1]
        url = GUI.url_dict[filename]
        req = request.Request(url, headers={'User-Agent': 'Mozilla/5.0'})
        u = request.urlopen(req)
        f = open(filename, 'wb')

        file_size_dl = 0
        block_sz = 8192
        while GUI.run_download:
            buffer = u.read(block_sz)
            if not buffer:
                break
            file_size_dl += len(buffer)
            f.write(buffer)
            status = "Downloading " + filename + " [%3.2f%%]" % (file_size_dl * 100. / GUI.size_dict[filename])
            GUI.box.erase()
            GUI.box.addstr(0, 0, status, GUI.high_light_text)
            GUI.box.addstr(curses.LINES - 1, 0, "C:Cancel Download", GUI.high_light_text)
            GUI.screen.refresh()
            GUI.box.refresh()

        f.close()
        GUI.run_download = False
        GUI.key = curses.KEY_DOWN
        GUI.update_screen()



import curses

try:
    from urllib import request
except ImportError:
    import urllib2 as request

from scrapy.crawler import CrawlerProcess

from music_scraper.gui import GUI
from music_scraper.threads import GuiThread
from music_scraper.spiders.music_spider import MusicSpider

try:
    input = raw_input
except NameError:
    pass


def start_gui(process):
    def create_ui(screen):
        GUI.screen = screen
        GUI.strings = []
        GUI.init_display()
        GUI.update_on_key()
        curses.nocbreak()
        curses.echo()
        curses.endwin()
        GUI.gui_stopped = True

    curses.wrapper(create_ui)
    process.stop()


def main():
    process = CrawlerProcess({'LOG_ENABLED': False})
    message = ''
    while message == '':
        message = input("Give me something to start with - (Example: kabali song download ) : ")
    s = request.quote(message)
    MusicSpider.start_urls = [
        "http://www.google.com/search?q=" + s,
    ]
    process.crawl(MusicSpider)
    thread = GuiThread(process, start_gui)
    thread.start()
    process.start()
    if len(GUI.strings) == 0 and not GUI.gui_stopped:
        GUI.box.erase()
        GUI.box.addstr(0, 0, "No Results Found... Try with Some other keywords.", GUI.high_light_text)
        GUI.box.addstr(curses.LINES - 1, 0, "ESC:Exit", GUI.high_light_text)
        GUI.box.addstr(curses.LINES - 1, curses.COLS // 2, "ENTR:Download", GUI.high_light_text)
        GUI.screen.refresh()
        GUI.box.refresh()



import threading


class GuiThread(threading.Thread):
    def __init__(self, process, func):
        threading.Thread.__init__(self)
        self.process = process
        self.func = func

    def run(self):
        self.func(self.process)


class DownloadThread(threading.Thread):
    def __init__(self, func):
        threading.Thread.__init__(self)
        self.func = func

    def run(self):
        self.func()



from os import path

import io

import music_scraper

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup
    from pkgutil import walk_packages


    def _find_packages(path='.', prefix=''):
        yield prefix
        prefix += "."
        for _, name, is_package in walk_packages(path,
                                                 prefix,
                                                 onerror=lambda x: x):
            if is_package:
                yield name


    def find_packages():
        return list(_find_packages(music_scraper.__path__, music_scraper.__name__))

here = path.abspath(path.dirname(__file__))

with io.open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


setup(name='music_scraper',
      version='1.0.0',
      install_requires=['scrapy >= 1.1.1'],
      description='Gets Songs from the web and allows users to download the same',
      long_description=long_description,
      url='https://github.com/srivatsan-ramesh/Music-Scrapper',
      author='srivatsan-ramesh',
      author_email='sriramesh4@gmail.com',
      license='MIT',
      packages=find_packages(),
      entry_points={
          'console_scripts': ['music-scraper=music_scraper.main:main'],
      },
      zip_safe=False)



import cgi

import scrapy

from music_scraper.gui import GUI

items = []


def is_standard_website(href):
    if 'google' not in href and 'youtube' not in href and 'twitter' not in href and 'facebook' not in href:
        return True
    else:
        return False


class MusicSpider(scrapy.Spider):
    name = "music"

    def parse(self, response):
        for href in response.xpath("//a/@href").extract():
            if href[:7] == '/url?q=' and is_standard_website(href):
                url = href[7:].split('&')[0]
                yield scrapy.Request(url, meta={'download_maxsize': 2097152}, callback=self.parse_dir_contents)

    def parse_dir_contents(self, response):
        for sel in response.xpath('//a/@href').extract():
            url = response.urljoin(sel)
            yield scrapy.Request(url, method='HEAD', callback=self.parse_final_contents)

    def parse_final_contents(self, response):
        if response.headers['Content-Type'].decode('UTF-8') == 'audio/mpeg':
            _, params = cgi.parse_header(response.headers.get('Content-Disposition', '').decode('UTF-8'))
            filename = params['filename']
            while True:
                if GUI.url_dict.get(filename) is None:
                    GUI.url_dict[filename] = response.url
                    GUI.size_dict[filename] = int(response.headers['Content-Length'].decode('UTF-8'))
                    break
                else:
                    split_file = filename.split('.')
                    filename = ''.join(split_file[:-1] + ['_.'] + split_file[-1:])
            GUI.strings += [filename]
            GUI.refresh_values()
            GUI.update_screen()
